@@ -15,7 +15,7 @@ def flip(ar, verbose=False):
         fx.printmsg('flipping radargram...')
     return ar.T[::-1].T
 
-def reducex(ar, by=1, chnum=1, number=1, verbose=False):
+def reducex(ar, header, by=1, chnum=1, number=1, verbose=False):
     """
     Reduce the number of traces in the array by a number. Not the same as :py:func:`stack` since it doesn't sum adjacent traces, however :py:func:`stack` uses it to resize the array prior to stacking.
 
@@ -34,7 +34,7 @@ def reducex(ar, by=1, chnum=1, number=1, verbose=False):
             fx.printmsg('%s/%s reducing %sx%s chunk by a factor of %s...' % (chnum, number, ar.shape[0], ar.shape[1], by))
     return ar[:,::by]
 
-def stack(ar, stack='auto', verbose=False):
+def stack(ar, header, stack='auto', verbose=False):
     """
     Stacking algorithm. Stacking is the process of summing adjacent traces in order to reduce noise --- the thought being that random noise around zero will cancel out and data will either add or subtract, making it easier to discern.
 
@@ -69,7 +69,7 @@ def stack(ar, stack='auto', verbose=False):
             fx.printmsg('stacking %sx %s...' % (stack, am))
         i = list(range(stack))
         l = list(range(int(ar.shape[1]/stack)))
-        arr = np.copy(reducex(ar=ar, by=stack, verbose=verbose))
+        arr = np.copy(reducex(ar=ar, by=stack, header=header, verbose=verbose))
         for s in l:
             arr[:,s] = arr[:,s] + ar[:,s*stack+1:s*stack+stack].sum(axis=1)
     else:
@@ -78,7 +78,29 @@ def stack(ar, stack='auto', verbose=False):
             pass
         else:
             fx.printmsg('WARNING: no stacking applied. this can result in very large and awkwardly-shaped figures.')
-    return arr, stack
+
+    if header['rh_nchan'] == 1:
+        # this is a hack to prevent the rhf_spx vars from being changed multiple times
+        # when stacking a multichannel file. this will go away when each array has its
+        # own header and thus each can be dealt with individually.
+        if header['rhf_sps'] != 0:
+            header['rhf_sps'] = header['rhf_sps'] / stack
+        if header['rhf_spm'] != 0:
+            header['rhf_spm'] = header['rhf_spm'] / stack
+    else:
+        try:
+            if header['spx_updates']:
+                pass
+            else:
+                if header['rhf_sps'] != 0:
+                    header['rhf_sps'] = header['rhf_sps'] / stack
+                if header['rhf_spm'] != 0:
+                    header['rhf_spm'] = header['rhf_spm'] / stack
+                header['spx_updates'] = True
+        except KeyError:
+            header['spx_updates'] = False
+
+    return header, arr, stack
 
 def distance_normalize(header, ar, gps, verbose=False):
     """
@@ -93,7 +115,7 @@ def distance_normalize(header, ar, gps, verbose=False):
     :rtype: header (:py:class:`dict`), radar array (:py:class:`numpy.ndarray`), gps (False or :py:class:`pandas.DataFrame`)
 
     """
-    if ar[2] == []:
+    if gps.empty:
         if verbose:
             fx.printmsg('no gps information for distance normalization')
     else:
@@ -108,7 +130,7 @@ def distance_normalize(header, ar, gps, verbose=False):
         newdf = pd.DataFrame(index=pd.date_range(start=start, periods=ar.shape[1], freq=str(nanosec_samp_rate)+'N', tz='UTC'))
         norm_vel = pd.concat([norm_vel, newdf], axis=1).interpolate('time').bfill()
         del newdf
-        norm_vel = norm_vel.round().astype(int, casting='unsafe')
+        norm_vel = norm_vel.round().astype(int, errors='ignore')#, casting='unsafe')
 
         try:
             rm = int(round(ar.shape[1] / (norm_vel.shape[0] - ar.shape[1])))
@@ -126,13 +148,15 @@ def distance_normalize(header, ar, gps, verbose=False):
         on, i = 0, 0
         for c in np.array_split(ar, nvm, axis=1):
             # takes (array, [transform values to broadcast], axis)
-            p = np.repeat(c, norm_vel['normalized'].astype(int, casting='unsafe').values[on:on+c.shape[1]], axis=1)
-            p = reducex(p, by=nvm, chnum=i, number=nvm, verbose=verbose)
+            p = np.repeat(c, norm_vel['normalized'].astype(int, errors='ignore').values[on:on+c.shape[1]], axis=1)
+            p = reducex(p, header=header, by=nvm, chnum=i, number=nvm, verbose=verbose)
             proc = np.concatenate((proc, p), axis=1)
             on = on + c.shape[1]
             i += 1
         if verbose:
+            fx.printmsg('total GPS distance: %.2f m' % gps['meters'].iloc[-1])
             fx.printmsg('replacing old traces per meter value of %s with %s' % (header['rhf_spm'],
                                                                             ar.shape[1] / gps['meters'].iloc[-1]))
         header['rhf_spm'] = proc.shape[1] / gps['meters'].iloc[-1]
+        header['rhf_sps'] = 0
     return header, proc, gps
