@@ -29,7 +29,7 @@ required flags:
 optional flags:
      OPTION     |      ARGUMENT       |       FUNCTIONALITY
 -o, --output    | file:  /dir/f.ext   |  output file. if not set, will be named similar to input
--f, --format    | string, eg. "csv"   |  output format (csv is the only working format currently)
+-f, --format    | string, eg. "csv"   |  output format (CSV and DZT are the only working formats currently available from the command line)
 -p, --plot      | +integer or "auto"  |  plot size. will be x inches high or "auto". default: 10. see also -D to set DPI
 -D, --dpi       | positive integer    |  set the plot DPI for figure making. defaults to 150
 -T, --titleoff  |                     |  turn the plot title off (useful for figure making)
@@ -39,6 +39,7 @@ optional flags:
 -n, --noshow    |                     |  suppress matplotlib popup window and simply save a figure (useful for multi-file processing)
 -c, --colormap  | string, eg. "Greys" |  specify the colormap (https://matplotlib.org/users/colormaps.html#grayscale-conversion)
 -g, --gain      | positive float      |  gain constant (higher=greater contrast, default: 1)
+-A, --absval    |                     |  Displays the absolute value of the vertical gradient of the array when plotting. Good for displaying faint array features.
 -r, --bgr       | +integer or zero    |  horizontal background removal (useful to remove ringing). zero=full width; positive=window size (after stacking)
 -R, --reverse   |                     |  reverse (flip array horizontally)
 -w, --dewow     |                     |  trinomial dewow algorithm
@@ -63,6 +64,7 @@ naming scheme for exports:
     Dw          |  Dewow filter
     Bp70-130    |  triangular FIR filter applied from 70 to 130 MHz
     G30         |  30x contrast gain
+    Abs         |  Color scale represents absolute value of vertical gradient
     Z10.20.7.5  |  zoom from 10-20 axis units on the x-axis and 5-7 on the z-axis
 ''' % (__version__, u'\U0001F12F', author, year, affil)
 
