@@ -6,6 +6,9 @@ This module contains a number of variables that readgssi needs to perform physic
 
 MINHEADSIZE = 1024 # absolute minimum total header size
 PAREASIZE = 128 # fixed info header size
+RGPSSIZE = 9 # GPS record size
+GPSAREASIZE = RGPSSIZE * 2
+INFOAREASIZE = MINHEADSIZE - PAREASIZE - GPSAREASIZE
 
 TZ = pytz.timezone('UTC')
 
@@ -26,7 +29,7 @@ UNIT = {
     7: 'StructureScan Mini',
     8: 'SIR 4000',
     9: 'SIR 30',
-    10: 'SIR 30', # 10 is undefined in documentation but SIR 30 according to Lynn's DZX
+    10: 'unknown system type',
     11: 'unknown system type',
     12: 'UtilityScan DF',
     13: 'HS',
@@ -68,6 +71,8 @@ ANT = {
     '5103A': 400,
     '50400': 400,
     '50400S': 400,
+    '3102 500MHZ': 500,
+    '3102': 500, # presumed to exist
     '800': 800,
     'D50800': 800,
     '3101': 900,
